#!/usr/bin/env python
# -*- coding: utf-8 -*-

import maya.cmds as cmds

# Global vars
import hmScripts.hm_global_variables as global_vars

# naming conventions
import hmScripts.hm_global_variables as nc

# Creation modules
import hmScripts.Main.hmSelectionHandling as hmSelect
import hmScripts.Main.hmNodeConnector as hmNodesConnection

# This class will be used for creating, connecting and setting nodes attributes
import hmScripts.Main.hmBaseObject as hmNode
from hmScripts.hm_global_variables import language
from hmScripts.Main.hmUtility import reference_copy

messages = language.values['perform_new_operation']

def perform_new_operation(ui_class_object):
    '''
    Create a new operation from the input taken from the current Maya selection.
    Get the options of creation from the current UI instance.

         ________________
        | ui_class_object| ----
         ----------------      |
                               | 	 ________________
                                ----|     Perform    |
                               | 	 ----------------
         ________________      |
        | node connector | ----
         ----------------

    Args:
        ui_class_object: The instance of the current HardMeshOperationOptionsUI

    Returns:
        None
    '''
    createOperation = True

    # Init the sel handling
    selection = hmSelect.HmSelectionHandling()
    ok_button = messages['ok_btn']

    # ---------------------------------------------------------------------------------------------------------------- #
    # If we support multimesh
    if global_vars.MULTIMESH:
        # Set custom messages
        selection.setErrorMessageIfZero(messages['min_select_mess'])
        selection.setErrorIfSmallThanMin(messages['min_select_mess'])

        # Validate the selection and put the result on a var
        sel = selection.validate(2, None, 'mesh')

        meshA = sel[0]
        meshB = sel[1:]

    else:
        # Set custom messages
        selection.setErrorMessageIfZero('Please select 2 meshes to operate on.')
        selection.setErrorIfSmallThanMin('Please select 2 meshes to operate on.')
        selection.setErrorMessageIfOutLimit('Please select only 2 objects. You can have multiple shells for each mesh.')

        # Validate the selection and put the result on a var
        sel = selection.validate(2, 2, 'mesh')

        meshA = sel[0]
        meshB = sel[1]
    # ---------------------------------------------------------------------------------------------------------------- #

    # Count how many out mesh are used as input
    found_out_mesh = []

    # Check that the meshes are not already involved in an operation
    for mesh in sel:

        mesh_obj = hmNode.HmBaseObject(mesh)

        hm_type = mesh_obj.get_object_type()

        # If the current mesh has hm identification attributes
        if hm_type:

            if hm_type == nc.hm_in_mesh:
                # We should do further verifications to see if it is really connected
                if global_vars.AVOID_BRANCHING:
                    cmds.confirmDialog(title=messages['nested_network_title'],
                                       message=messages['nested_network_in'].format(mesh),
                                       button=[ok_button],
                                       defaultButton=ok_button)

                    createOperation = False

            # Avoid that more than one hm_out_mesh are used in the creation of a new operation
            if hm_type == nc.hm_out_mesh:

                # Store it for counting
                found_out_mesh.append(mesh)

                # We should do further verifications to see if it is really connected

                # --- MISSING

                if len(found_out_mesh) > 1:

                    if global_vars.AVOID_BRANCHING:
                        cmds.confirmDialog(title=messages['nested_network_title'],
                                           message=messages['nested_network_out'].format(mesh),
                                           button=[ok_button],
                                           defaultButton=ok_button)
                        createOperation = False

                # Check if this mesh is in between a chain
                if mesh_obj.get_shape().get_connected('outMesh', 'out', node_type='hmOffsetCurveNode'):

                    cmds.confirmDialog(title=messages['nested_network_title'],
                                       message=messages['nested_network_mid'].format(mesh),
                                       button=[ok_button],
                                       defaultButton=ok_button)
                    createOperation = False

    # Proceed in the creation only if the var is true
    if createOperation:
        # ==== Get the interface values
        ui_class_object.getValuesFromInterface()

        # #==== Create the node network

        # Initialize the instance
        hmSetup = hmNodesConnection.HmOperationConnect([meshA] + [x for x in meshB], 1,
                                                       ui_class_object.hmopt_out_shader_val[1])

        # Set the options
        hmSetup.operation_val = ui_class_object.hmopt_operation_val[1] - 1
        hmSetup.smooth_val_a = ui_class_object.hmopt_smooth_val_a[1]
        hmSetup.smooth_val_b = ui_class_object.hmopt_smooth_val_b[1]
        hmSetup.offset_val = ui_class_object.hmopt_offset_val[1]
        hmSetup.geo_bias_val = ui_class_object.hmopt_geo_bias_val[1]
        hmSetup.sect_val = ui_class_object.hmopt_sections_val[1]
        hmSetup.chord_dist = ui_class_object.hmopt_chord_dist_val[1]
        hmSetup.split_refine = ui_class_object.hmopt_refine_val[1] - 1
        hmSetup.inter_blend = ui_class_object.hmopt_smooth_profile_val[1]
        hmSetup.weight_val_a = ui_class_object.hmopt_weight_a_val[1]
        hmSetup.weight_val_b = ui_class_object.hmopt_weight_b_val[1]
        hmSetup.use_ramp_val = ui_class_object.hmopt_active_ramped_filler_val[1]
        hmSetup.ramp_values = ui_class_object.hmopt_profile_curve_val[1]
        hmSetup.ramp_offset_val = ui_class_object.hmopt_ramp_offset_val[1]
        hmSetup.ramp_scale_val = ui_class_object.hmopt_ramp_scale_val[1]
        # Set the custom shader name for this operation
        hmSetup.hm_custom_shader = ui_class_object.current_custom_shader

        # Create the setup
        hmSetup.create_connections()



def perform_split_multi_operation():
    selection = hmSelect.HmSelectionHandling()

    # Set custom messages
    selection.setErrorMessageIfZero('Please select 2 meshes to operate on.')
    selection.setErrorIfSmallThanMin('Please select 2 meshes to operate on.')
    selection.setErrorMessageIfOutLimit('Please select only 2 objects. You can have multiple shells for each mesh.')

    # Validate the selection and put the result on a var
    meshes = selection.validate(2, None, 'mesh')
    ref_meshes = reference_copy(meshes)

    # Parent the reference copies, add an attribute on the orig meshes for managing the visibility of the sub meshes
    for orig_m, ref_m in zip(meshes, ref_meshes):
        # ref_m.parent = orig_m

        orig_m_obj = hmNode.HmBaseObject(orig_m)
        orig_m_obj.add_attribute(global_vars.sub_ref_attr_name, at='enum', en='off:on', k=True)

        driven = ref_m.add_group('DRIVEN')

        orig_m_obj.connect_attribute(global_vars.sub_ref_attr_name, driven, 'visibility')

        decomp = hmNode.HmBaseObject(cmds.createNode('decomposeMatrix'))
        orig_m_obj.connect_attribute('worldMatrix', decomp, 'inputMatrix')
        decomp.connect_attribute('outputTranslate', driven, 'translate')
        decomp.connect_attribute('outputRotate', driven, 'rotate')
        decomp.connect_attribute('outputScale', driven, 'scale')



    # Subtraction
    #
    subtract_oper = hmNodesConnection.HmOperationConnect(meshes, 1, global_vars.default_shader)
    subtract_oper.operation_val = 1
    subtract_oper.create_connections()

    # Intersection
    #
    intersect_oper = hmNodesConnection.HmOperationConnect(ref_meshes, 1, global_vars.default_shader)
    intersect_oper.operation_val = 3
    intersect_oper.create_connections()